/******************************************************************/
/* make vector consistent                                         */
/******************************************************************/
#include <stdlib.h>
#include "mpi.h"

void  AverageVector(int left_neighbour,int right_neighbour,
                    int lower_neighbour,int upper_neighbour,
                    int x_int_loc,int  y_int_loc, double *vec)
{
   int i, nx, ny, length, *number, *snumbuf, *rnumbuf0, *rnumbuf1;
   double *sbuf, *rbuf0, *rbuf1;
   MPI_Status status;

   nx = x_int_loc+1;
   ny = y_int_loc+1;
   
   /* compute maximum of nx and ny */
   if (nx>ny)
      length = nx;
   else
      length = ny;

   /* allocate buffers for sending and receiving data */
   if (!(sbuf=malloc(length*sizeof(*sbuf))))
      MPI_Abort(MPI_COMM_WORLD, 1);
   if (!(rbuf0=malloc(length*sizeof(*rbuf0))))
      MPI_Abort(MPI_COMM_WORLD, 1);
   if (!(rbuf1=malloc(length*sizeof(*rbuf1))))
      MPI_Abort(MPI_COMM_WORLD, 1);
   if (!(number=malloc(nx*ny*sizeof(*number))))
      MPI_Abort(MPI_COMM_WORLD, 1);
   memset(number,0,nx*ny*sizeof(*number));
   if (!(snumbuf=malloc(length*sizeof(*snumbuf))))
      MPI_Abort(MPI_COMM_WORLD, 1);
   if (!(rnumbuf0=malloc(length*sizeof(*rnumbuf0))))
      MPI_Abort(MPI_COMM_WORLD, 1);
   if (!(rnumbuf1=malloc(length*sizeof(*rnumbuf1))))
      MPI_Abort(MPI_COMM_WORLD, 1);

   for(i=0;i<nx*ny;i++)
      number[i] = 1;
   
   /****************************************************************/
   /* lower and upper boundaries                                   */
   /****************************************************************/

   /* to lower neighbour */
   if ((lower_neighbour>-1))
   {
      for (i=0;i<nx;i++)
         sbuf[i] = vec[i];
      MPI_Send(sbuf, length, MPI_DOUBLE, lower_neighbour, 0,  MPI_COMM_WORLD);
   }
    /* from upper  neighbour */
   if ((upper_neighbour>-1))
      MPI_Recv(rbuf0, length, MPI_DOUBLE, upper_neighbour, 0,  MPI_COMM_WORLD, &status);
   
   /* to  upper neighbour */
   if ((upper_neighbour>-1))
   {
       for (i=0;i<nx;i++)
         sbuf[i] = vec[nx*(ny-1)+i];
       MPI_Send(sbuf, length, MPI_DOUBLE, upper_neighbour, 0,  MPI_COMM_WORLD);     
   }
   /* from lower neighbour */
   if ((lower_neighbour>-1))
      MPI_Recv(rbuf1, length, MPI_DOUBLE, lower_neighbour, 0,  MPI_COMM_WORLD, &status);
 
   /* update values */
   if ((upper_neighbour>-1))
   {
      for (i=0;i<nx;i++)
      {
         vec[nx*(ny-1)+i] += rbuf0[i];
         number[nx*(ny-1)+i]++;
      }
   }

   if ((lower_neighbour>-1))
   {
      for (i=0;i<nx;i++)
      {
         vec[i] += rbuf1[i];
         number[i]++;
      }
   }

   /****************************************************************/
   /* left and right boundaries                                    */
   /****************************************************************/

  /* to left neighbour */
   if ((left_neighbour>-1))
   {
       for (i=0;i<ny;i++)
         sbuf[i] = vec[i*nx];
       MPI_Send(sbuf, length, MPI_DOUBLE, left_neighbour, 0,  MPI_COMM_WORLD);     
       for (i=0;i<ny;i++)
         snumbuf[i] = number[i*nx];
       MPI_Send(snumbuf, length, MPI_INT, left_neighbour, 0,  MPI_COMM_WORLD);     
   }
   /* from right neighbour */
   if ((right_neighbour>-1))
   {
      MPI_Recv(rbuf0, length, MPI_DOUBLE, right_neighbour, 0,  MPI_COMM_WORLD, &status);
      MPI_Recv(rnumbuf0, length, MPI_INT, right_neighbour, 0,  MPI_COMM_WORLD, &status);
   }
   /* to right neighbour */
   if ((right_neighbour>-1))
   {
       for (i=0;i<ny;i++)
         sbuf[i] = vec[(i+1)*nx-1];
       MPI_Send(sbuf, length, MPI_DOUBLE, right_neighbour, 0,  MPI_COMM_WORLD);     
       for (i=0;i<ny;i++)
         snumbuf[i] = number[(i+1)*nx-1];
       MPI_Send(snumbuf, length, MPI_INT, right_neighbour, 0,  MPI_COMM_WORLD);     
   }
   /* from left neighbour */
   if ((left_neighbour>-1))
   {
      MPI_Recv(rbuf1, length, MPI_DOUBLE, left_neighbour, 0,  MPI_COMM_WORLD, &status);
      MPI_Recv(rnumbuf1, length, MPI_INT, left_neighbour, 0,  MPI_COMM_WORLD, &status);
   }
   /* update values */
   if ((right_neighbour>-1))
   {
      for (i=0;i<ny;i++)
      {
         vec[(i+1)*nx-1] += rbuf0[i];
         number[(i+1)*nx-1] += rnumbuf0[i];
      }
   }
   if ((left_neighbour>-1))
   {
      for (i=0;i<ny;i++)
      {
         vec[i*nx] += rbuf1[i];
         number[i*nx]+= rnumbuf1[i];
      }
   }
  
   /* compute average */
   for (i=0;i<nx*ny;i++)
   {
      if (number[i]>1)
      {
         vec[i] /= (number[i]);
         /* printf("vec %g %d \n" ,vec[i] ,number[i]);*/
      }
   }

   free(sbuf);
   free(rbuf0);
   free(rbuf1);
   free(number);
   free(snumbuf);
   free(rnumbuf0);
   free(rnumbuf1);

   return;
}
