#include "headers.h"

/******************************************************************/
/* set Dirichlet values                                           */
/* the matrix and the rhs will change, they have to be still      */
/* stored inconsistently after the manipulations !!!              */
/******************************************************************/

void SetDirichletValues(int left_neighbour,int right_neighbour,
                        int lower_neighbour, int upper_neighbour,
                        int x_int_loc, int y_int_loc,
                        int x_subdoms, int y_subdoms,
                        double x0, double y0,
                        int *RowPtr, int *KCol, double *matrix_entries,
                        double *sol, double *rhs)
{
   int i, j, k, start, end, ii, nx, ny, startj, endj, row, find, cond;
   double x, y;
   
   nx = x_int_loc+1;
   ny = y_int_loc+1;
      
   /****************************************************************/
   /* there is no neighbour on the lower side                      */
   /****************************************************************/
   if ((lower_neighbour==-1))
   {      
      for (i=0;i<nx-1;i++)
      {
         start = RowPtr[i];
         end = RowPtr[i+1];
         for (j=start;j<end;j++)
         {
            if (KCol[j]==i)
               matrix_entries[j] = 1; 
            else
               matrix_entries[j] = 0; 
         }
         ComputeCoordinates(x0, y0, i, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
        
         rhs[i] = BoundCond(x,y);  
         sol[i] = rhs[i];
      }

      /* special treatment for last point */
      /* if there is a right neighbour, then set matrix_entries and rhs */
      /* to zero to ensure inconsistent storage */
      /* sol has to be stored consistently */
      cond = 1;
      start = RowPtr[nx-1];
      end = RowPtr[nx];
      /* x - coordinate is 1 */
      if (right_neighbour==-1)
      {
         for (j=start;j<end;j++)
         {
            if (KCol[j]==nx)
               matrix_entries[j] = 1; 
            else
               matrix_entries[j] = 0; 
         }
         ComputeCoordinates(x0, y0, nx-1, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
         
         rhs[nx-1] = BoundCond(x,y); 
         sol[nx-1] = rhs[nx-1];
         cond = 1;
      }
      else
      {
         for (j=start;j<end;j++)
            matrix_entries[j] = 0; 
         rhs[nx-1] = 0; 
         ComputeCoordinates(x0, y0, nx-1, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
         
         sol[nx-1] = BoundCond(x,y);         
      }

      /* condensate Dirichlet values */
      for (i=0;i<nx-1+cond;i++)
      {
         start = RowPtr[i];
         end = RowPtr[i+1];
         /* for each column of row i */
         for (j=start;j<end;j++)
         {
            /* this is the row corresponding to the column */
            row = KCol[j];
            if (row==i)
               continue;
            startj = RowPtr[row];
            endj =  RowPtr[row+1];
            /* go through this row */
            for (k=startj;k<endj;k++)
            {
               find = 0;
               /* find matrix entry a(row,i) */
               if (KCol[k]==i)
               {
                  rhs[row] -= matrix_entries[k] * sol[i];
                  matrix_entries[k] = 0;
                  find ++;
                  break;
               }
            }
            if (!find) 
               printf("not find1 %d !!! \n",cond);
         }            
      }
   }

  /****************************************************************/
  /* there is no neighbour on the upper side                      */
  /****************************************************************/
   if ((upper_neighbour==-1))
   {      
      for (i=nx*(ny-1);i<nx*ny-1;i++)
      {
         start = RowPtr[i];
         end = RowPtr[i+1];
         for (j=start;j<end;j++)
         {
            if (KCol[j]==i)
               matrix_entries[j] = 1; 
            else
               matrix_entries[j] = 0; 
         }
         ComputeCoordinates(x0, y0, i, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
        
         rhs[i] = BoundCond(x,y);           
         sol[i] = rhs[i];           
      }
      
      /* special treatment for last point */
      /* if there is a right neibour, then set matrix_entries and rhs */
      /* to zero to ensure inconsistent storage */
      cond = 1;
      start = RowPtr[nx*ny-1];
      end = RowPtr[nx*ny];
      /* x - coordinate is 1 */
      if (right_neighbour==-1)
      {
         for (j=start;j<end;j++)
         {
            if (KCol[j]==nx*ny-1)
               matrix_entries[j] = 1; 
            else
               matrix_entries[j] = 0; 
         }
         ComputeCoordinates(x0, y0, nx*ny-1, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
         
         rhs[nx*ny-1] = BoundCond(x,y); 
         sol[nx*ny-1] = rhs[nx*ny-1];         
         cond = 1;
      }
      else
      {
         for (j=start;j<end;j++)
            matrix_entries[j] = 0; 
         rhs[nx*ny-1] = 0;               
         ComputeCoordinates(x0, y0, nx*ny-1, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
         sol[nx*ny-1] = BoundCond(x,y); 
     }

      /* condensate Dirichlet values */
      for (i=nx*(ny-1);i<nx*ny-1+cond;i++)
      {
         start = RowPtr[i];
         end = RowPtr[i+1];
         /* for each column of row i */
         for (j=start;j<end;j++)
         {
            /* this is the row corresponding to the column */
            row = KCol[j];
            if (row==i)
               continue;
            startj = RowPtr[row];
            endj =  RowPtr[row+1];
            /* go through this row */
            for (k=startj;k<endj;k++)
            {
               find = 0;
               /* find matrix entry a(row,i) */
               if (KCol[k]==i)
               {
                  rhs[row] -= matrix_entries[k] * sol[i];
                  matrix_entries[k] = 0;
                  find ++;
                  break;
            }
            }
            if (!find) 
               printf("not find2 %d !!! \n",cond);
         }            
      }
   }
   /****************************************************************/
   /* there is no neighbour on the left side                       */
   /****************************************************************/
   if ((left_neighbour==-1))
   {      
      for (ii=0;ii<ny-1;ii++)
      {
         i = ii * nx;
         start = RowPtr[i];
         end = RowPtr[i+1];
         for (j=start;j<end;j++)
         {
            if (KCol[j]==i)
               matrix_entries[j] = 1; 
            else
               matrix_entries[j] = 0; 
         }
         ComputeCoordinates(x0, y0, i, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
        
         rhs[i] = BoundCond(x,y); 
         sol[i] = rhs[i];          
      }

      /* special treatment for last point */
      /* if there is an upper neibour, then set matrix_entries and rhs */
      /* to zero to ensure inconsistent storage */
      cond = 1;
      start = RowPtr[(ny-1)*nx];
      end = RowPtr[(ny-1)*nx+1];
      /* x - coordinate is 1 */
      if (upper_neighbour==-1)
      {
         for (j=start;j<end;j++)
         {
            if (KCol[j]==(ny-1)*nx)
               matrix_entries[j] = 1; 
            else
               matrix_entries[j] = 0; 
         }
         ComputeCoordinates(x0, y0, (ny-1)*nx, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
         
         rhs[nx*(ny-1)] = BoundCond(x,y); 
         sol[nx*(ny-1)] = rhs[nx*(ny-1)]; 
         cond = 1;
      }
      else
      {
         for (j=start;j<end;j++)
            matrix_entries[j] = 0; 
         rhs[nx*(ny-1)] = 0;               
         ComputeCoordinates(x0, y0, (ny-1)*nx, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
         sol[nx*(ny-1)] = BoundCond(x,y); 
      }

      /* condensate Dirichlet values */
      for (ii=0;ii<ny-1+cond;ii++)
      {
         i = ii * nx;
         start = RowPtr[i];
         end = RowPtr[i+1];
         /* for each column of row i */
         for (j=start;j<end;j++)
         {
            /* this is the row corresponding to the column */
            row = KCol[j];
            if (row==i)
               continue;
            startj = RowPtr[row];
            endj =  RowPtr[row+1];
            /* go through this row */
            for (k=startj;k<endj;k++)
            {
               find = 0;
               /* find matrix entry a(row,i) */
               if (KCol[k]==i)
               {
                  rhs[row] -= matrix_entries[k] * sol[i];
                  matrix_entries[k] = 0;
                  find ++;
                  break;
               }
            }
            if (!find) 
               printf("not find3 !!! \n");
         }            
      }
   }

   /****************************************************************/
   /* there is no neighbour on the right side                      */
   /****************************************************************/
   if ((right_neighbour==-1))
   {      
      for (ii=0;ii<ny-1;ii++)
      {
         i = (ii+1) * nx -1 ;
         start = RowPtr[i];
         end = RowPtr[i+1];
         for (j=start;j<end;j++)
         {
            if (KCol[j]==i)
               matrix_entries[j] = 1; 
            else
               matrix_entries[j] = 0; 
         }
         ComputeCoordinates(x0, y0, i, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
        
         rhs[i] = BoundCond(x,y);           
         sol[i] = rhs[i];           
      }
      
      /* special treatment for last point */
      /* if there is an upper neibour, then set matrix_entries and rhs */
      /* to zero to ensure inconsistent storage */
      cond = 1;
      start = RowPtr[nx*ny-1];
      end = RowPtr[nx*ny];
      /* x - coordinate is 1 */
      if (upper_neighbour==-1)
      {
         for (j=start;j<end;j++)
         {
            if (KCol[j]==nx*ny-1)
               matrix_entries[j] = 1; 
            else
               matrix_entries[j] = 0; 
         }
         ComputeCoordinates(x0, y0, nx*ny-1, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
         
         rhs[nx*ny-1] = BoundCond(x,y);
         sol[nx*ny-1] = rhs[nx*ny-1];
         cond = 1;
      }
      else
      {
         for (j=start;j<end;j++)
            matrix_entries[j] = 0; 
         rhs[nx*ny-1] = 0;               
         ComputeCoordinates(x0, y0, nx*ny-1, x_subdoms, y_subdoms,  
                            x_int_loc, y_int_loc, &x, &y);
         
         sol[nx*ny-1] = BoundCond(x,y);
      }

      /* condensate Dirichlet values */
      for (ii=0;ii<ny-1+cond;ii++)
      {
         i = (ii+1) * nx -1 ;
         start = RowPtr[i];
         end = RowPtr[i+1];
         /* for each column of row i */
         for (j=start;j<end;j++)
         {
            /* this is the row corresponding to the column */
            row = KCol[j];
            if (row==i)
               continue;
            startj = RowPtr[row];
            endj =  RowPtr[row+1];
            /* go through this row */
            for (k=startj;k<endj;k++)
            {
               find = 0;
            /* find matrix entry a(row,i) */
               if (KCol[k]==i)
               {
                  rhs[row] -= matrix_entries[k] * sol[i];
                  matrix_entries[k] = 0;
                  find ++;
                  break;
               }
            }
            if (!find) 
               printf("not find4 !!! \n");
         }            
      }
   }
  return;
}
